##############################################################################
#
# Copyright (c) Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
import re, unittest, os

from zope.testing import doctest, renormalizing
from zope.app.testing import placelesssetup, ztapi

from ore.wsgiapp.app import Application

test_zcml_contents = open( os.path.join( os.path.dirname(__file__), 'test.zcml'  ) )

class TestApplication( Application ):
    """ a really simple containerish application """
    
class AppView( object ):
    """ a simple view we register for the application """
    def __init__( self,context, request):
        self.context = context 
        self.request = request
        
    def __call__( self ):
        return "Hello World"
    
def test_suite():
    
    ns = dict(
        component = ztapi,
        test_zcml_contents = test_zcml_contents 
        )
        
    return unittest.TestSuite((
        doctest.DocFileSuite(
            'readme.txt',
            checker = renormalizing.RENormalizing([
                (re.compile('at 0x[0-9a-f]+'), 'at <SOME ADDRESS>'),
                ]),
            ),
        ))    



##############################################################################
#
# Copyright (c) 2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: $
"""

from zope import interface
from zope.component.interfaces import IObjectEvent

class IApplication( interface.Interface ):
    """
    Application Root to publish
    """

class IWSGIApplicationCreatedEvent( IObjectEvent ):
    
    app = interface.Attribute(u"Published Application")

class WSGIApplicationCreatedEvent( object ):
    
    interface.implements( IWSGIApplicationCreatedEvent )
    
    def __init__( self, object ):
        self.object = object


##############################################################################
#
# Copyright (c) 2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: $
"""

from zope.app.publication.browser import BrowserPublication
from zope.app.publication.interfaces import IRequestPublicationFactory, IBrowserRequestFactory

from zope.publisher.browser import BrowserRequest
from zope import component, interface

import interfaces

class Publication( BrowserPublication ):

    def getApplication( self, request ):
        app = component.getUtility( interfaces.IApplication )
        return app

class BrowserFactory( object ):

    interface.implements( IRequestPublicationFactory )

    def canHandle( self, environment ):
        return True

    def __call__(self):
        request_class = component.queryUtility(
            IBrowserRequestFactory, default=BrowserRequest)
        return request_class, Publication
    



##############################################################################
#
# Copyright (c) 2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: $
"""

import os
import sys
import code
import zope.app.wsgi
import zope.app.debug

import wsgi

def application_factory(global_conf, zcml='site.zcml', devmode='off'):
    zcml_conf = os.path.join(global_conf['here'], zcml)
    devmode = ( devmode.lower() in ('true', 'True', 'on') and True) or False
    return wsgi.getWSGIApplication(zcml_conf, devmode)




import os
from setuptools import setup, find_packages

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

setup(name='ore.wsgiapp',
      version='0.3.2',
      description="glue for wsgi zope3 apps w/o zodb",
      long_description=( read('src','ore','wsgiapp','README.txt')
                         + '\n\n' +
                         read('changes.txt')
                         ),
      keywords='',
      author='Kapil Thangavelu',
      author_email='kapil.foss@gmail.com',
      license='ZPL',
      # Get more from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=['Programming Language :: Python',
                   'Environment :: Web Environment',
                   'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
                   'Framework :: Zope3',
                   ],
      packages=find_packages('src'),
      package_dir = {'':'src'},
      namespace_packages = ['ore',],
      include_package_data=True,
      zip_safe=False,
      install_requires=['setuptools',
                        'zope.publisher',
                        'zope.traversing',
                        'zope.event',
                        'zope.app.wsgi>=3.4.0',
                        'zope.app.appsetup',
                        'zope.app.zcmlfiles',
                        'zope.testing',
                        ],
      entry_points = """
      [paste.app_factory]
      main = ore.wsgiapp.startup:application_factory
      """
      )




from zope import interface
from zope.app.component import site
from zope.app.container.sample import SampleContainer
from zope.traversing.interfaces import IContainmentRoot

import interfaces

class BaseApplication( site.SiteManagerContainer ):

    interface.implements( interfaces.IApplication, IContainmentRoot )
    
class Application( SampleContainer, BaseApplication ):
    pass



##############################################################################
#
# Copyright (c) Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""A WSGI Application wrapper for zope

"""

import logging

from zope.event import notify
from zope import component
from zope.interface import implements
from zope.publisher.publish import publish

from zope.app.appsetup import appsetup
from zope.app.publication.httpfactory import HTTPPublicationRequestFactory
from zope.app.wsgi import interfaces

from interfaces import IApplication, WSGIApplicationCreatedEvent

class WSGIPublisherApplication(object):
    """A WSGI application implementation for the zope publisher

    Instances of this class can be used as a WSGI application object.

    The class relies on a properly initialized request factory.
    """
    implements(interfaces.IWSGIApplication)

    def __init__(self, db=None, factory=HTTPPublicationRequestFactory):
        self.requestFactory = factory(db)

    def __call__(self, environ, start_response):
        """See zope.app.wsgi.interfaces.IWSGIApplication"""
        request = self.requestFactory(environ['wsgi.input'], environ)

        # Let's support post-mortem debugging
        handle_errors = environ.get('wsgi.handleErrors', True)

        request = publish(request, handle_errors=handle_errors)
        response = request.response

        # Start the WSGI server response
        start_response(response.getStatusString(), response.getHeaders())

        # Return the result body iterable.
        return response.consumeBodyIter()


class PMDBWSGIPublisherApplication(WSGIPublisherApplication):

    def __call__(self, environ, start_response):
        environ['wsgi.handleErrors'] = False

        # Call the application to handle the request and write a response
        try:
            app =  super(PMDBWSGIPublisherApplication, self)
            return app.__call__(environ, start_response)
        except Exception, error:
            import sys, pdb
            print "%s:" % sys.exc_info()[0]
            print sys.exc_info()[1]
            #import zope.security.management
            #zope.security.management.restoreInteraction()
            try:
                pdb.post_mortem(sys.exc_info()[2])
                raise
            finally:
                pass #zope.security.management.endInteraction()


def config( zcml_conf, devmode, features=()):
    # Insert the devmode feature, if turned on
    if devmode:
        features += ('devmode',)
        logging.warning("Developer mode is enabled: this is a security risk "
            "and should NOT be enabled on production servers. Developer mode "
            "can be turned off in etc/zope.conf")
    # Execute the ZCML configuration.
    appsetup.config( zcml_conf, features=features)
    # notify the app we're configured
    app = component.getUtility( IApplication )
    notify( WSGIApplicationCreatedEvent( app ) )
    return None

def getWSGIApplication(configfile, schemafile=None, features=(),
                       requestFactory=HTTPPublicationRequestFactory):
    config(configfile, schemafile, features)
    return WSGIPublisherApplication( None, requestFactory)
